==> gnue-forms_0.5.12.orig.tar.gz: gnue-forms-0.5.12/src/uidrivers/win32/ToolBar.py <==
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2005 Free Software Foundation
#
# $Id: ToolBar.py 7718 2005-07-09 20:48:31Z btami $
#
# DESCRIPTION:
# A PyWin32 based user interface driver for GNUe forms.
#
# NOTES:
#

import os
import sys
import array
import struct

import win32api
import win32con
import win32gui
import commctrl

from gnue.common import events
from gnue.common.apps import GConfig

from gnue.forms.uidrivers._commonGuiToolkit.ToolBar import ToolBar as _BaseToolBar
from gnue.forms.uidrivers.win32.widgets._base import Win32Window
from gnue.forms.uidrivers.win32.common import getNextId, textEncode

winver = sys.getwindowsversion()
XP = winver[0]==5 and winver[1]==1 and winver[3]==2

class ToolBar(_BaseToolBar):

  # Create the menu
  def init(self):
    hinst = win32api.GetModuleHandle(None)
    style = win32con.WS_CHILD | commctrl.TBSTYLE_TOOLTIPS | commctrl.TBSTYLE_FLAT #|win32con.WS_BORDER 
    styleEx = 0
    self.toolbar = mainToolBar = Win32Window(self.driver, styleEx, commctrl.TOOLBARCLASSNAME, "GNUe toolbar",
      style, 0, 0, 0, 0,
      self.container, getNextId(), hinst)

    style = win32con.WS_CHILD | win32con.SS_SUNKEN
    Win32Window(self.driver, 0, 'STATIC', '', style, 0, 30, 9999, 2, self.toolbar)

    if XP:
      self.himl = himl = win32gui.ImageList_Create(24, 24, commctrl.ILC_COLOR32, 0, 3)
    else:
      iconpath = GConfig.getInstalledBase('form_images', 'common_images')
      iconloc = os.path.join(iconpath, 'forms/default', 'toolbar-24x24.bmp')
      if not os.path.isfile(iconloc):
        print "*** WARNING: Cannot add 'toolbar-24x24.bmp' to toolbar"
        return
      else:
        himl = win32gui.ImageList_LoadImage(0, iconloc, 24, 0, win32api.RGB(192,192,192), win32con.IMAGE_BITMAP,
          win32con.LR_LOADFROMFILE | win32con.LR_SHARED | win32con.LR_CREATEDIBSECTION)
          
    tbab = struct.pack("ii", 0, himl)
    win32gui.SendMessage(mainToolBar.GetHwnd(), commctrl.TB_SETIMAGELIST, 0, himl)

    self._htt = win32gui.SendMessage(mainToolBar.GetHwnd(), commctrl.TB_GETTOOLTIPS, 0, 0)
    win32gui.SendMessage(self._htt, commctrl.TTM_SETMAXTIPWIDTH , 0, 250)
    
    self._buttonCount = 0
    win32gui.SendMessage(mainToolBar.GetHwnd(), commctrl.TB_AUTOSIZE, 0, 0)
    self.toolbar.Show()

    return self.toolbar

  # Add a menu item (action)
  def addAction(self, name, userAction):
    parent = self.toolbar
    label = name

    # Create an event binding in windows
    id = getNextId()
    event = 'request' + userAction.event

    if XP:
      iconloc = userAction.getIconLocation(format="bmp", size="24x24")
    
      # Set the action icon if available
      if iconloc:
        icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_SHARED
        him = win32gui.LoadImage(0, iconloc, win32con.IMAGE_BITMAP, 24, 24, icon_flags)
        win32gui.ImageList_Add(self.himl, him, 0)
      else:
        print "** WARNING: Cannot add '%s' to toolbar; no icon" % userAction.event
        return

    self.container.Connect(id, lambda l=self.driver, e=event, f=self.form: l.dispatchEvent(events.Event(e,_form=f)))

#    TBBUTTON stru (iBitmap, idCommand, fsState, fsStyle, dwData, iString)
    tbb=struct.pack("iibbli", self._buttonCount, id, commctrl.TBSTATE_ENABLED, commctrl.TBSTYLE_BUTTON, 0, 0)
    win32gui.SendMessage(parent.GetHwnd(), commctrl.TB_ADDBUTTONS, 1, tbb)
    # i have to count myself 'coz TB_BUTTONCOUNT counts separators too...
    self._buttonCount += 1

#    TOOLINFO stru (cbSize, uFlags, hwnd, uId, rect, hinst, lpszText, lParam)
    buff = array.array('c', '%s \0x00' % (textEncode(userAction.description) or ''))
    addrText = buff.buffer_info()[0]
    format = "IIiIllllili"
    size = struct.calcsize(format)
    ti = struct.pack(format, size, 0, self.toolbar.GetHwnd(), id, 0, 0, 0, 0, 0, addrText, 0)
    win32gui.SendMessage(self._htt, commctrl.TTM_UPDATETIPTEXT , 0, ti)

    return (parent, id)


  # Add a separator
  def addSeparator(self):
      tbb=struct.pack("iibbli", 0, 0, commctrl.TBSTATE_ENABLED, commctrl.TBSTYLE_SEP, 0, 0)
      win32gui.SendMessage(self.toolbar.GetHwnd(), commctrl.TB_ADDBUTTONS, 1, tbb)

  # Enable a menu item
  def enableItem(self, item):
    parent, id = item
    win32gui.SendMessage(self.toolbar.GetHwnd(), commctrl.TB_ENABLEBUTTON, id, 1)

  # Disable a menu item
  def disableItem(self, item):
    parent, id = item
    win32gui.SendMessage(self.toolbar.GetHwnd(), commctrl.TB_ENABLEBUTTON, id, 0)

  def startingItem(self, item):
    return
    self.toolbar.ToggleTool(item,1)

  def endingItem(self, item):
    return
    self.toolbar.ToggleTool(item,0)

==> gnue-forms_0.5.13.orig.tar.gz: gnue-forms-0.5.13/src/uidrivers/win32/ToolBar.py <==
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2005 Free Software Foundation
#
# $Id: ToolBar.py 7988 2005-09-24 17:04:40Z btami $
#
# DESCRIPTION:
# A PyWin32 based user interface driver for GNUe forms.
#
# NOTES:
#

import os
import sys
import array
import struct

import win32api
import win32con
import win32gui
import commctrl

from gnue.common import events
from gnue.common.apps import GConfig

from gnue.forms.uidrivers._commonGuiToolkit.ToolBar import ToolBar as _BaseToolBar
from gnue.forms.uidrivers.win32.widgets._base import Win32Window
from gnue.forms.uidrivers.win32.common import getNextId, textEncode

winver = sys.getwindowsversion()
XP = winver[0]==5 and winver[1]==1 and winver[3]==2

class ToolBar(_BaseToolBar):

  # Create the menu
  def init(self):
    hinst = win32api.GetModuleHandle(None)
    style = win32con.WS_CHILD | commctrl.TBSTYLE_TOOLTIPS | commctrl.TBSTYLE_FLAT #|win32con.WS_BORDER 
    styleEx = 0
    self.toolbar = mainToolBar = Win32Window(self.driver, styleEx, commctrl.TOOLBARCLASSNAME, "GNUe toolbar",
      style, 0, 0, 0, 0,
      self.container, getNextId(), hinst)

    style = win32con.WS_CHILD | win32con.SS_SUNKEN
    Win32Window(self.driver, 0, 'STATIC', '', style, 0, 30, 9999, 2, self.toolbar)

    if XP:
      self.himl = himl = win32gui.ImageList_Create(24, 24, commctrl.ILC_COLOR32, 0, 3)
    else:
      iconpath = GConfig.getInstalledBase('form_images', 'common_images')
      iconloc = os.path.join(iconpath, 'forms/default', 'toolbar-24x24.bmp')
      if not os.path.isfile(iconloc):
        print "*** WARNING: Cannot add 'toolbar-24x24.bmp' to toolbar"
        return
      else:
        himl = win32gui.ImageList_LoadImage(0, iconloc, 24, 0, win32api.RGB(192,192,192), win32con.IMAGE_BITMAP,
          win32con.LR_LOADFROMFILE | win32con.LR_SHARED | win32con.LR_CREATEDIBSECTION)
          
    tbab = struct.pack("ii", 0, himl)
    win32gui.SendMessage(mainToolBar.GetHwnd(), commctrl.TB_SETIMAGELIST, 0, himl)

    self._htt = win32gui.SendMessage(mainToolBar.GetHwnd(), commctrl.TB_GETTOOLTIPS, 0, 0)
    win32gui.SendMessage(self._htt, commctrl.TTM_SETMAXTIPWIDTH , 0, 250)
    
    self._buttonCount = 0
    win32gui.SendMessage(mainToolBar.GetHwnd(), commctrl.TB_AUTOSIZE, 0, 0)
    self.toolbar.Show()

    return self.toolbar

  # Add a menu item (action)
  def addAction(self, name, userAction):
    parent = self.toolbar
    label = name

    # Create an event binding in windows
    id = getNextId()
    event = 'request' + userAction.event

    if XP:
      iconloc = userAction.getIconLocation(format="bmp", size="24x24")
    
      # Set the action icon if available
      if iconloc:
        icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_SHARED
        him = win32gui.LoadImage(0, iconloc, win32con.IMAGE_BITMAP, 24, 24, icon_flags)
        win32gui.ImageList_Add(self.himl, him, 0)
      else:
        print "** WARNING: Cannot add '%s' to toolbar; no icon" % userAction.event
        return

    if userAction.canToggle:
      self.container.Connect(id, 
        lambda l=self.driver,
              e=event,
              te='request%s' % userAction.canToggle,
              tb=parent.GetHwnd(),
              f=self.form: l.dispatchEvent(
                  (win32gui.SendMessage(tb, commctrl.TB_ISBUTTONCHECKED, id, 0) and e or te),_form=f))
      check = 1
      style = commctrl.TBSTYLE_CHECK

    else:
      self.container.Connect(id, 
        lambda l=self.driver, e=event, f=self.form: l.dispatchEvent(e,_form=f))
      check = 0
      style = commctrl.TBSTYLE_BUTTON

#    TBBUTTON stru (iBitmap, idCommand, fsState, fsStyle, dwData, iString)
    tbb=struct.pack("iibbli", self._buttonCount, id, commctrl.TBSTATE_ENABLED, style, 0, 0)
    win32gui.SendMessage(parent.GetHwnd(), commctrl.TB_ADDBUTTONS, 1, tbb)
    # i have to count myself 'coz TB_BUTTONCOUNT counts separators too...
    self._buttonCount += 1

#    TOOLINFO stru (cbSize, uFlags, hwnd, uId, rect, hinst, lpszText, lParam)
    buff = array.array('c', '%s \0x00' % (textEncode(userAction.description) or ''))
    addrText = buff.buffer_info()[0]
    format = "IIiIllllili"
    size = struct.calcsize(format)
    ti = struct.pack(format, size, 0, self.toolbar.GetHwnd(), id, 0, 0, 0, 0, 0, addrText, 0)
    win32gui.SendMessage(self._htt, commctrl.TTM_UPDATETIPTEXT , 0, ti)

    return (parent, id, check)


  # Add a separator
  def addSeparator(self):
      tbb=struct.pack("iibbli", 0, 0, commctrl.TBSTATE_ENABLED, commctrl.TBSTYLE_SEP, 0, 0)
      win32gui.SendMessage(self.toolbar.GetHwnd(), commctrl.TB_ADDBUTTONS, 1, tbb)

  # Enable a menu item
  def enableItem(self, item):
    parent, id, check = item
    win32gui.SendMessage(self.toolbar.GetHwnd(), commctrl.TB_ENABLEBUTTON, id, 1)

  # Disable a menu item
  def disableItem(self, item):
    parent, id, check = item
    win32gui.SendMessage(self.toolbar.GetHwnd(), commctrl.TB_ENABLEBUTTON, id, 0)

  def startingItem(self, item):
    parent, id, check = item
    if check:
      win32gui.SendMessage(self.toolbar.GetHwnd(), commctrl.TB_CHECKBUTTON, id, 1)
    else:
      win32gui.SendMessage(self.toolbar.GetHwnd(), commctrl.TB_PRESSBUTTON, id, 1)

  def endingItem(self, item):
    parent, id, check = item
    if check:
      win32gui.SendMessage(self.toolbar.GetHwnd(), commctrl.TB_CHECKBUTTON, id, 0)
    else:
      win32gui.SendMessage(self.toolbar.GetHwnd(), commctrl.TB_PRESSBUTTON, id, 0)
